'  LANSI.BAS               The FSA ANSI-Driver             LANSI.BAS
'
'                             Laleh's ANSI
'
'                   Written by Quinn Tyler Jackson
'
'     Copyright (C)1993 by Quinn Tyler Jackson.  ALL RIGHTS RESERVED.
'
'  USAGE RIGHTS:
'
'  Although Quinn Tyler Jackson reserves all rights to LANSI.BAS, he grants
'  others the right to use it in whole or in part as long as any product
'  that uses it explicitly includes in its documentation or opening screen
'  (this left to the discretion of the individual programmer) the notice:
'
'       "ANSI emmulation provided by Laleh's ANSI (C)1993 by QTJ."
'
'  DEDICATION:
'
'  This driver is dedicated to my wife Laleh, who has tolerated my sitting
'  at a yucky computer terminal for thousands of hours with little or no
'  direct return to her.
'
'  TECHINICAL NOTES: (31 OCT 93)
'
'  LANSI.BAS was written as an exercise in my investigation into the
'  the complexities of Finite State Automata.  (FSA's)  ANSI graphics
'  are particularly suitable to an FSA model, since they rely on a finite
'  set of commands and use a type of Reverse Polish Notation (RPN).
'  Reverse Polish Notation lends itself amazingly well to efficient
'  implementations of Finite State systems.
'
'  This driver includes non-ANSI "ANSI" music support, but this support
'  can be turned off by setting the global variable GV.Music is set to
'  FALSE (0).  With this variable set to FALSE, LANSI treats music strings
'  in exactly the same manner a typical ANSI.SYS driver would: it prints
'  them to the screen.  This option has been added for ANSI purists.
'
'  Note that the music supported here can be full background.  A 6.5 minute
'  song can be downloaded as a series of music sequences in about 5 seconds
'  and then played over then next 6.5 minutes in the background on a typical
'  2400 baud modem using the system I have implemented here.  This system
'  buffers up to 200 "lines" of music sequences.  Many terminals don't allow
'  full background music, since they are not written in BASIC and therefore
'  have to emmulate the PLAY metalanguage.  BASIC has direct access to
'  "MB", so this is no problem if double-buffering is used.
'
'  Also, LANSI.BAS supports a subset of the ANSI keyboard redefinition
'  capabilities, unlike many other ANSI emmulators, which totally ignore
'  this part of the ANSI standard.  Support for keyboard redefinition can
'  be toggled on and off as well.  (The only format allowed is:
'
'   ESC[nn;"some string"p
'
'  Where nn is an ASCII value and "some string" is a quote-deliminated
'  literal string.)
'
'  This code is, for the most part, raw and undocumented.  I've done the
'  hard coding; it's up to you to figure out what I've done.  My belief
'  is simple in this regard:  if you cannot decipher the code without
'  detailed English comments and remarks, then you probably should NOT
'  be changing the code!
'
'  If you can understand the raw code, then you might want to go and
'  tweak it.  I've found that I do best to leave code I don't "quite"
'  understand well-enough-alone.  Be warned that FSA systems are quick,
'  but prone to nastiness when incorrectly tweaked.  That's the nature
'  of the finite state paradigm.
'
'  Also, you will notice the term VisiPlex from time to time.  Just ignore
'  it.  This will follow much later, and is included in LANSI.BAS just for
'  future compatibility.
'
'
'  Hope you find LANSI.BAS useful!
'
'
'  Quinn Tyler Jackson (1:153/918.1@fidonet.org)
'
'
'
'  APPENDIX:
'
'  I use a particularly systematic naming convention when I program in
'  QuickBASIC:
'
'  1)   SUB PROGRAMS that I intend to have others call, I prefix with
'       a simple "sub".  These are PUBLIC SUBs that can be invoked fairly
'       simply without disrupting the whole module.  In short, those SUBs
'       that begin "sub..." belong to anyone to use.
'
'  2)   SUB PROGRAMS that I do not intend to allow others to call, I prefix
'       with an "sqj".  These are PRIVATE SUBs that should not be invoked
'       if the second programmer does not fully understand their workings,
'       since to do so might disrupt the whole functioning of the module.
'       In short, those SUBs that begin "sqj..." belong to Quinn Jackson
'       for his exclusive use, and any other use should only be after a
'       thorough examination of the code involved has taken place.
'
'  3)   FUNCTIONs (of which there are none in this program) take the
'       PUBLIC prefix "fun..." and the PRIVATE prefix "fqj...."
'
'
'  Also, I try to name all SUBs using a standard convention:
'
'       <prefix><Object>[Object Modifier]<Method>
'
'  An example of this:
'
'       sqjCursorLocate
'        ^    ^     ^
'        |    |     |
'    <prefix> |     |
'          <Object> |
'                   |
'               <Method>
'
'  This should immediately indicate that this SUB is private, was
'  coded by Quinn Jackson, and locates the cursor.  Since all SUBs are
'  named in this fashion, one can immediately identify the purpose of
'  other SUBs using the same system:
'
'       sqjScreenModeSet
'


DECLARE SUB subByteInterpret (Byte AS INTEGER)
DECLARE SUB sqjControlCodeReact (code AS INTEGER)
DECLARE SUB sqjCursorDown (RowsDown AS INTEGER)
DECLARE SUB sqjCursorLeft (ColsLeft AS INTEGER)
DECLARE SUB sqjCursorLocate (X AS INTEGER, Y AS INTEGER)
DECLARE SUB sqjCursorRestore ()
DECLARE SUB sqjCursorRight (ColsRight AS INTEGER)
DECLARE SUB sqjCursorSave ()
DECLARE SUB sqjCursorUp (RowsUp AS INTEGER)
DECLARE SUB sqjEOLErase ()
DECLARE SUB sqjIntPush ()
DECLARE SUB sqjKeyboardMap (KeyCode AS INTEGER, Redefinition AS STRING)
DECLARE SUB sqjLinewrapDisable ()
DECLARE SUB sqjMusicPlay (Score AS STRING)
DECLARE SUB sqjPRINT (Byte AS INTEGER, Level AS INTEGER)
DECLARE SUB sqjRemoteCursorSet (X AS INTEGER, Y AS INTEGER)
DECLARE SUB sqjScreenAttrReact ()
DECLARE SUB sqjScreenAttrSet (Attribute AS INTEGER)
DECLARE SUB sqjScreenClear (Param AS INTEGER)
DECLARE SUB sqjScreenModeSet (Mode AS INTEGER)
DECLARE SUB sqjScreenWidthSet (Columns AS INTEGER)
DECLARE SUB sqjStateReact (Byte AS INTEGER)
DECLARE SUB sqjStateTableInit ()
DECLARE SUB sqjStringPush ()
DECLARE SUB sqjSystemReqReact (Request AS INTEGER)
DECLARE SUB sqjTokenTableInit ()
DECLARE SUB sqjVisiPlexComReact (ComType AS INTEGER, VisiCommand AS STRING)
DECLARE SUB sqjBarPush (Score AS STRING)
DECLARE SUB sqjPLAY (Score AS STRING)

DEFINT A-Z

TYPE GlobalVarType
    STATE AS INTEGER            ' What state is the FSA in?
  
    X AS INTEGER                ' Cursor ROW
    Y AS INTEGER                ' Cursor COLUMN
    OldX AS INTEGER             ' For saving ROW with $e[s
    OldY AS INTEGER             ' For saving COLUMN with $e[s
    RemoteX AS INTEGER          ' The remote's ROW
    RemoteY AS INTEGER          ' The remote's COLUMN

    ScreenHeight AS INTEGER     ' What is the height of our screen?
    ScreenWidth AS INTEGER      ' What is the width of our screen?

    Bold AS INTEGER             ' Bold attribute
    Blink AS INTEGER            ' Blink attribute
    Reversed AS INTEGER         ' Reversed attribute
    Concealed AS INTEGER        ' Concealed attribute
  
    DesBackspace AS INTEGER     ' Set to TRUE if <BACKSPACE> is destructive
    ExpandTab AS INTEGER        ' Set to TRUE if <TAB> is expanded
    TabStep AS INTEGER          ' Number of spaces to expand 1 tab
    LineWrap AS INTEGER         ' Set to true if in linewrap mode
    CursorVis AS INTEGER        ' Set to 1 if cursor is visible
    Music AS INTEGER            ' Set to FALSE if in ANSI only mode
    Speaker AS INTEGER          ' Set to FALSE if sound accepted but not heard
    BeepHz AS INTEGER           ' Beep tone in Hertz
    BeepDur AS INTEGER          ' Duration of Beep in ticks
   
    MapActive AS INTEGER

    ForeColor AS INTEGER
    BackColor AS INTEGER
  
    SavedFlag AS INTEGER        ' Has a $e[s been previously executed?
  
    VisiPlex AS INTEGER         ' Are we in VisiPlex mode
    VisiVersion AS INTEGER      ' If so, what is version of other?
END TYPE


CONST TRUE = -1
CONST FALSE = NOT TRUE

CONST LOW = FALSE
CONST HIGH = TRUE

CONST ANSI.F.BLACK = 30
CONST ANSI.F.RED = 31
CONST ANSI.F.GREEN = 32
CONST ANSI.F.YELLOW = 33
CONST ANSI.F.BLUE = 34
CONST ANSI.F.MAGENTA = 35
CONST ANSI.F.CYAN = 36
CONST ANSI.F.WHITE = 37
CONST ANSI.B.BLACK = 40
CONST ANSI.B.RED = 41
CONST ANSI.B.GREEN = 42
CONST ANSI.B.YELLOW = 43
CONST ANSI.B.BLUE = 44
CONST ANSI.B.MAGENTA = 45
CONST ANSI.B.CYAN = 46
CONST ANSI.B.WHITE = 47

CONST STATE.NORMAL = 0
CONST STATE.READ.ESC = 1
CONST STATE.IN.ANSI = 2
CONST STATE.IN.INT.PARAM = 3
CONST STATE.READ.SEMICOLON = 4
CONST STATE.READ.ANSI.COMMAND = 5

CONST STATE.READ.OPEN.QUOTE = 6
CONST STATE.IN.STRING.LITERAL = 7
CONST STATE.READ.CLOSE.QUOTE = 8
CONST STATE.READ.CONTROL.CODE = 9
CONST STATE.ERROR.RESET.ANSI = 10

CONST STATE.IN.MUSIC = 11

CONST STATE.INTEGER.PUSH = 12       ' push integer to integer stack
CONST STATE.STRING.PUSH = 13        ' push string to string stack




'   TOKEN TYPES:
    '
    '   e = ESC
CONST TOKEN.ESC = 1
    '   [ = [
CONST TOKEN.BRACKET = 2
    '   0 = 0,1,2,3,4,5,6,7,8,9
CONST TOKEN.DIGIT = 3
    '   ; = ;
CONST TOKEN.SEMICOLON = 4
    '   H = H,f,A,B,C,D,s,u,J,K,m,h,l,p,n
CONST TOKEN.ANSI.COMMAND = 5
    '   " = "
CONST TOKEN.QUOTE = 6
    '   < = ASCII code less than 32
CONST TOKEN.CONTROL.CODE = 7
    '   A = Standard A-Z
CONST TOKEN.ASCII = 8
    '   M = M
CONST TOKEN.MUSIC.STRING.START = 9
    '   # = ^N
CONST TOKEN.ANSI.MUSIC = 10

TokenTableData:
      
'        ASCII   Type     ASCII   Type
DATA      "",    01,      "[",    02
DATA      "0",    03,      "1",    03
DATA      "2",    03,      "3",    03
DATA      "4",    03,      "5",    03
DATA      "6",    03,      "7",    03
DATA      "8",    03,      "9",    03
DATA      ";",    04,      "H",    05
DATA      "f",    05,      "A",    05
DATA      "B",    05,      "C",    05
DATA      "D",    05,      "s",    05
DATA      "u",    05,      "J",    05
DATA      "K",    05,      "m",    05
DATA      "h",    05,      "l",    05
DATA      "p",    05,      "R",    05
DATA      "n",    05,      "",    10
DATA      "M",    09,      "",    07
DATA      "	",    07,      "",    07
DATA      "",    07,      "",    07
DATA      "",    07,      "",    07
DATA      "",    07
DATA      " ",    -1

StateShiftTableData:

' WARNING:
'
'   Any tweaking of this table may be FATAL to the working
'   of this driver!  Unless you ABSOLUTELY understand what
'   you are doing, please DO NOT twiddle these bits!

'            e   [   0   ;   H   "   <   A   M   #
DATA   00,  01, 00, 00, 00, 00, 00, 09, 00, 00, 00
DATA   01,  00, 02, 00, 00, 00, 00, 00, 00, 00, 00
DATA   02,  00, 10, 03, 04, 05, 06, 10, 10, 11, 10
DATA   03,  10, 10, 03, 12, 05, 10, 10, 10, 10, 10
DATA   04,  10, 10, 12, 10, 10, 06, 10, 10, 10, 10
DATA   06,  07, 07, 07, 07, 07, 08, 07, 07, 07, 10
DATA   07,  07, 07, 07, 07, 07, 08, 07, 07, 07, 07
DATA   08,  10, 10, 10, 11, 05, 10, 10, 10, 10, 10
DATA   09,  01, 00, 00, 00, 00, 00, 09, 00, 00, 00
DATA   11,  10, 10, 11, 10, 11, 10, 10, 11, 11, 05

DIM SHARED GV AS GlobalVarType  ' make COMMON to allow access to
                                ' certain features from other modules

DIM SHARED TokenTable(255) AS INTEGER
DIM SHARED StateTable(13, 10) AS INTEGER
DIM SHARED KeyTable(255) AS STRING
DIM SHARED MapActive(255) AS INTEGER

DIM SHARED CharBuffer AS STRING
DIM SHARED IntStack(10) AS INTEGER
DIM SHARED StrStack(10) AS STRING
DIM SHARED IntPtr AS INTEGER
DIM SHARED StrPtr AS INTEGER

DIM SHARED MusicBuffer(200) AS STRING
DIM SHARED BarPtr AS INTEGER
DIM SHARED TopPtr AS INTEGER

sqjTokenTableInit   ' THESE TABLES **MUST** BE INITIALIZED FOR
sqjStateTableInit   ' THIS DRIVER TO WORK AT ALL!!!!!

DO
    K$ = INKEY$
    IF K$ <> "" THEN
        subByteInterpret ASC(K$)
    END IF
LOOP

END



MusicHandler:
    BarPtr = BarPtr + 1
    TotChar = TotChar + LEN(MusicBuffer(BarPtr))
    SELECT CASE BarPtr
        CASE 201
            BarPtr = 1
        CASE TopPtr
            PLAY "MF" + MusicBuffer(BarPtr)
            OverFlag = TRUE
            BarPtr = 0
            TopPtr = 0
            PLAY OFF
    END SELECT
    IF TopPtr > 1 THEN
        PLAY MusicBuffer(BarPtr)
    END IF
    MusicBuffer(BarPtr) = ""
    RETURN

SUB sqjBarPush (Score AS STRING)

TopPtr = TopPtr + 1
IF TopPtr = 201 THEN
    NotFirst = TRUE
    TopPtr = 1
END IF

MusicBuffer(TopPtr) = Score

IF TopPtr = 1 AND NOT NotFirst THEN
    PLAY "MBT255N0N0N0T120"
END IF

END SUB

SUB sqjControlCodeReact (code AS INTEGER)

CONST CONT.CTRL.D = 4
CONST CONT.CTRL.E = 5
CONST CONT.CTRL.G = 7
CONST CONT.BACKSPACE = 8
CONST CONT.TAB = 9
CONST CONT.PAGEFEED = 12
CONST CONT.CTRL.S = 19
CONST CONT.CTRL.X = 24

SELECT CASE code
    CASE CONT.CTRL.E
        sqjCursorUp 1

    CASE CONT.CTRL.X
        sqjCursorDown 1

    CASE CONT.CTRL.D
        sqjCursorRight 1

    CASE CONT.CTRL.S
        sqjCursorLeft 1

    CASE CONT.CTRL.G
        IF GV.Speaker THEN
            SOUND GV.BeepHz, GV.BeepDur
        END IF

    CASE CONT.PAGEFEED
        sqjScreenClear 0
   
    CASE CONT.BACKSPACE
        IF GV.DesBackspace THEN
            IF GV.Y > 1 THEN
                GV.Y = GV.Y - 1
                sqjCursorLocate GV.X, GV.Y
                sqjPRINT 32, HIGH' Print a space
                GV.Y = GV.Y - 1
                sqjCursorLocate GV.X, GV.Y
            END IF
        END IF
   
    CASE CONT.TAB
        IF GV.ExpandTab THEN
            IF GV.Y + GV.TabStep < 79 THEN
                FOR i = 1 TO GV.TabStep
                    sqjPRINT 32, LOW
                NEXT i
            END IF
        ELSE
            sqjPRINT 32, HIGH
        END IF
END SELECT

END SUB

SUB sqjCursorDown (RowsDown AS INTEGER)

IF IntPtr = 0 THEN
    RowsDown = 1
END IF

TempX = GV.X + RowsDown

IF TempX > GV.ScreenHeight THEN
    TempX = GV.ScreenHeight
END IF

GV.X = TempX

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjCursorLeft (ColsLeft AS INTEGER)

IF IntPtr = 0 THEN
    ColsLeft = 1
END IF

TempY = GV.Y - ColsLeft
IF TempY < 1 THEN
    TempY = 1
END IF
   
GV.Y = TempY

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjCursorLocate (X AS INTEGER, Y AS INTEGER)

IF Y > GV.ScreenWidth THEN
    Y = 1
    IF GV.LineWrap THEN
        X = X + 1
    END IF
END IF

IF X > GV.ScreenHeight THEN
    EXIT SUB
END IF

GV.X = X
GV.Y = Y

LOCATE GV.X, GV.Y, GV.CursorVis

END SUB

SUB sqjCursorRestore

IF GV.SavedFlag THEN
    sqjCursorLocate GV.OldX, GV.OldY
END IF

END SUB

SUB sqjCursorRight (ColsRight AS INTEGER)

IF IntPtr = 0 THEN
    ColsRight = 1
END IF

TempY = GV.Y + ColsRight
IF TempY > GV.ScreenWidth THEN
    TempY = GV.ScreenWidth
END IF

GV.Y = TempY

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjCursorSave

GV.SavedFlag = TRUE

GV.OldX = GV.X
GV.OldY = GV.Y

END SUB

SUB sqjCursorUp (RowsUp AS INTEGER)

IF IntPtr = 0 THEN
    RowsUp = 1
END IF

TempX = GV.X - RowsUp

IF TempX < 1 THEN
    TempX = 1
END IF
   
GV.X = TempX

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjEOLErase

sqjCursorSave
FOR Ptr = GV.Y TO GV.ScreenWidth
    sqjPRINT 32, LOW
NEXT Ptr
sqjCursorRestore

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjIntPush

IF LEN(CharBuffer) THEN
    IntPtr = IntPtr + 1
    IntStack(IntPtr) = VAL(CharBuffer)
    CharBuffer = ""
END IF

END SUB

SUB sqjKeyboardMap (KeyCode AS INTEGER, Redefinition AS STRING)

SELECT CASE KeyCode
    CASE 777    ' Not a key!  This is the VisiPlex flag!
        IF LEFT$(Redefinition, 10) = "VisiPlex V" THEN
            GV.VisiPlex = TRUE
            GV.MapActive = FALSE
            GV.VisiVersion = VAL(MID$(Redefinition, 11))
        END IF
    CASE ELSE
        SELECT CASE GV.VisiPlex
            CASE TRUE
                sqjVisiPlexComReact KeyCode, Redefinition
            CASE FALSE
                IF KeyCode < 256 THEN
                    KeyTable(KeyCode) = Redefinition
                    MapActive(KeyCode) = TRUE
                END IF
        END SELECT
END SELECT

END SUB

SUB sqjLinewrapDisable

GV.LineWrap = FALSE

END SUB

SUB sqjMusicPlay (Score AS STRING)

SELECT CASE GV.Music
    CASE TRUE
        IF GV.Speaker THEN
            sqjPLAY Score
        END IF
    CASE FALSE
        FOR i = 1 TO LEN(Score)
            sqjPRINT ASC(MID$(Score, i, 1)), LOW
        NEXT i
        sqjPRINT 14, LOW
END SELECT

END SUB

SUB sqjPLAY (Score AS STRING)

Score = UCASE$(Score)

SELECT CASE LEFT$(Score, 2)
    CASE "MF"
        PLAY Score
    CASE "MB"
        ON PLAY(3) GOSUB MusicHandler
        PLAY ON
        sqjBarPush Score
    CASE ELSE
        sqjBarPush Score
END SELECT

END SUB

SUB sqjPRINT (Byte AS INTEGER, Level AS INTEGER)

SELECT CASE Level * MapActive(Byte) * GV.MapActive
    CASE 0
        SELECT CASE Byte
            CASE 13
                PRINT
                GV.X = GV.X + 1
                GV.Y = 1
            CASE ELSE
                PRINT CHR$(Byte);
                GV.Y = GV.Y + 1
        END SELECT
    CASE ELSE
        FOR i = 1 TO LEN(KeyTable(Byte))
            sqjPRINT ASC(MID$(KeyTable(Byte), i, 1)), LOW
        NEXT i
END SELECT

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjRemoteCursorSet (X AS INTEGER, Y AS INTEGER)

GV.RemoteX = X
GV.RemoteY = Y

END SUB

SUB sqjScreenAttrReact

SELECT CASE GV.Reversed
    CASE TRUE
        TempFore = GV.BackColor
        TempBack = GV.ForeColor
    CASE FALSE
        TempFore = GV.ForeColor
        TempBack = GV.BackColor
END SELECT


IF GV.Bold THEN
    TempFore = TempFore + 8
END IF

IF GV.Blink THEN
    TempFore = TempFore + 16
END IF

IF GV.Concealed THEN
    TempFore = TempBack
END IF

COLOR TempFore, TempBack

END SUB

SUB sqjScreenAttrSet (Attribute AS INTEGER)

SELECT CASE Attribute
    CASE 0
        GV.Bold = FALSE
        GV.Blink = FALSE
        GV.Reversed = FALSE
        GV.Concealed = FALSE
        GV.ForeColor = 7
        GV.BackColor = 0
    CASE 1
        GV.Bold = TRUE
    CASE 5
        GV.Blink = TRUE
    CASE 7
        GV.Reversed = TRUE
    CASE 8
        GV.Concealed = TRUE
    CASE ANSI.F.BLACK
        GV.ForeColor = 0
    CASE ANSI.F.RED
        GV.ForeColor = 4
    CASE ANSI.F.GREEN
        GV.ForeColor = 2
    CASE ANSI.F.YELLOW
        GV.ForeColor = 6
    CASE ANSI.F.BLUE
        GV.ForeColor = 1
    CASE ANSI.F.MAGENTA
        GV.ForeColor = 5
    CASE ANSI.F.CYAN
        GV.ForeColor = 3
    CASE ANSI.F.WHITE
        GV.ForeColor = 7
    CASE ANSI.B.BLACK
        GV.BackColor = 0
    CASE ANSI.B.RED
        GV.BackColor = 4
    CASE ANSI.B.GREEN
        GV.BackColor = 2
    CASE ANSI.B.YELLOW
        GV.BackColor = 6
    CASE ANSI.B.BLUE
        GV.BackColor = 2
    CASE ANSI.B.MAGENTA
        GV.BackColor = 5
    CASE ANSI.B.CYAN
        GV.BackColor = 3
    CASE ANSI.B.WHITE
        GV.BackColor = 7
END SELECT

END SUB

SUB sqjScreenClear (Param AS INTEGER)

GV.X = 1
GV.Y = 1
CLS
sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjScreenModeSet (Mode AS INTEGER)

SELECT CASE Mode
    CASE 0, 1
        sqjScreenWidthSet 40
    CASE 2, 3
        sqjScreenWidthSet 80
    CASE 7
        GV.LineWrap = TRUE
END SELECT

END SUB

SUB sqjScreenWidthSet (Columns AS INTEGER)

WIDTH Columns, 25

GV.ScreenWidth = Columns

END SUB

SUB sqjStateReact (Byte AS INTEGER)

SELECT CASE GV.STATE
    CASE STATE.NORMAL
        sqjPRINT Byte, HIGH
        
    CASE STATE.READ.ESC
        ' No need to "do" anything.

    CASE STATE.IN.ANSI
        ' No need to "do" anything.

    CASE STATE.IN.INT.PARAM
        CharBuffer = CharBuffer + CHR$(Byte)

    CASE STATE.READ.SEMICOLON
        sqjIntPush
        GV.STATE = STATE.IN.ANSI
        sqjStateReact 0

    CASE STATE.READ.ANSI.COMMAND
        SELECT CASE CHR$(Byte)
            CASE "H", "f"
                sqjIntPush
                SELECT CASE IntPtr
                    CASE 2
                        sqjCursorLocate IntStack(1), IntStack(2)
                    CASE 0
                        sqjCursorLocate 1, 1
                END SELECT
                IntPtr = 0
               
            CASE "A"
                sqjIntPush
                sqjCursorUp IntStack(1)
                IntPtr = 0
               
            CASE "B"
                sqjIntPush
                sqjCursorDown IntStack(1)
                IntPtr = 0
           
            CASE "C"
                sqjIntPush
                sqjCursorRight IntStack(1)
                IntPtr = 0
           
            CASE "D"
                sqjIntPush
                sqjCursorLeft IntStack(1)
                IntPtr = 0
           
            CASE "s"
                sqjCursorSave
           
            CASE "u"
                sqjCursorRestore
           
            CASE "J"
                sqjIntPush
                sqjScreenClear IntStack(1)
                IntPtr = 0
                          
            CASE "K"
                sqjEOLErase
           
            CASE "m"
                sqjIntPush
                FOR Ptr = 1 TO IntPtr
                    sqjScreenAttrSet IntStack(Ptr)
                NEXT Ptr
                sqjScreenAttrReact
                IntPtr = 0
           
            CASE "h"
                sqjIntPush
                sqjScreenModeSet IntStack(1)
                IntPtr = 0
           
            CASE "l"
                sqjIntPush
                sqjLinewrapDisable

            CASE "p"
                sqjKeyboardMap IntStack(1), StrStack(1)
                IntPtr = 0
                StrPtr = 0

            CASE "n"
                sqjIntPush
                sqjSystemReqReact IntStack(1)
                IntPtr = 0
           
            CASE ""
                sqjStringPush
                sqjMusicPlay StrStack(1)
                StrPtr = 0
            CASE "R"
                sqjIntPush
                sqjRemoteCursorSet IntStack(1), IntStack(2)
                IntPtr = 0

        END SELECT
        GV.STATE = STATE.NORMAL

    CASE STATE.READ.OPEN.QUOTE
   
    CASE STATE.IN.STRING.LITERAL
        CharBuffer = CharBuffer + CHR$(Byte)

    CASE STATE.READ.CLOSE.QUOTE
        sqjStringPush

    CASE STATE.READ.CONTROL.CODE
        sqjControlCodeReact Byte

    CASE STATE.ERROR.RESET.ANSI
        IntPtr = 0
        StrPtr = 0
        CharBuffer = ""
        sqjPRINT Byte, LOW
        GV.STATE = STATE.NORMAL

    CASE STATE.IN.MUSIC
        CharBuffer = CharBuffer + CHR$(Byte)

    CASE STATE.INTEGER.PUSH
        sqjIntPush
        GV.STATE = STATE.IN.ANSI

    CASE STATE.STRING.PUSH
        sqjStringPush
        GV.STATE = STATE.IN.ANSI

END SELECT

END SUB

SUB sqjStateTableInit

RESTORE StateShiftTableData

DIM Tkn(1 TO 10)

FOR STATE = 0 TO 10
    READ STATE
    READ Tkn(1), Tkn(2), Tkn(3), Tkn(4), Tkn(5), Tkn(6), Tkn(7), Tkn(8), Tkn(9), Tkn(10)
    FOR TokenType = 1 TO 10
        StateTable(STATE, TokenType) = Tkn(TokenType)
    NEXT
NEXT STATE

END SUB

SUB sqjStringPush

StrPtr = StrPtr + 1
StrStack(StrPtr) = CharBuffer
CharBuffer = ""

END SUB

SUB sqjSystemReqReact (Request AS INTEGER)

SELECT CASE Request
    CASE 6
        ' Request cursor position!
        ' Put code here that sends cursor position in format:
        '
        '       $e[xx;yyR

    CASE ELSE
        ' DOS's ANSI.SYS responds just as above!

END SELECT


END SUB

SUB sqjTokenTableInit

' Set some default start up values for the global system variables
' These will suffice for most purposes.

GV.X = 1
GV.Y = 1
GV.ScreenWidth = 80
GV.ScreenHeight = 24
GV.DesBackspace = TRUE
GV.ExpandTab = TRUE
GV.TabStep = 5
GV.LineWrap = TRUE
GV.ForeColor = 7
GV.BackColor = 0
GV.CursorVis = 1
GV.Music = TRUE
GV.Speaker = TRUE
GV.BeepHz = 300
GV.BeepDur = 3
GV.MapActive = TRUE

RESTORE TokenTableData

FOR i = 0 TO 255
    TokenTable(i) = TOKEN.ASCII
    KeyTable(i) = CHR$(i)
    MapActive(i) = FALSE
NEXT i

TokenTable(10) = 7
TokenTable(34) = 6

DO
    READ Char$, TokenType
    IF TokenType > 0 THEN
        TokenTable(ASC(Char$)) = TokenType
    ELSE
        EXIT DO
    END IF
LOOP

sqjCursorLocate GV.X, GV.Y

END SUB

SUB sqjVisiPlexComReact (ComType AS INTEGER, VisiCommand AS STRING)

' VisiPlex driver goes here and responds to the specific VisiPlex
' commands that will be standardized later!  Until then, you'll just
' have to wait.  This will most likely call an external VisiPlex protocol
' engine I will distribute as a VBDOS compatible library.
'
' It will be a simple matter of doing this here:
'
'       sqjVisiDriver ComType, VisiCommand
'
' AND BINGO!  Instant support!

END SUB

SUB subByteInterpret (Byte AS INTEGER)

GV.STATE = StateTable(GV.STATE, TokenTable(Byte))

sqjStateReact Byte

END SUB

